#include <gtk/gtk.h>
#include "cat.h"

#include "cb.h"
#include "ui.h"
#include "scan.h"
#include "savecat.h"
#include "utils.h"

#include "pixmaps/dir.xpm"
#include "pixmaps/open_dir.xpm"
#include "pixmaps/disk.xpm"
#include "pixmaps/open_disk.xpm"
#include "pixmaps/file.xpm"

/* global pixmaps */
GdkPixmap *img_dir;
GdkBitmap *img_dir_mask;
GdkPixmap *img_dir_open;
GdkBitmap *img_dir_open_mask;
GdkPixmap *img_disk;
GdkBitmap *img_disk_mask;
GdkPixmap *img_disk_open;
GdkBitmap *img_disk_open_mask;
GdkPixmap *img_file;
GdkBitmap *img_file_mask;

static void new_icon(GtkWidget *widget, char **data, GdkPixmap **pixmap, GdkBitmap **mask);
static GtkWidget *new_menu(GtkAccelGroup *accel, Ui *ui);

Ui *new_ui(Catalog *cat)
{
    Ui *ui;
    GtkWidget *vbox;
    GtkWidget *hpaned;
    GtkWidget *scroll;
    GtkAccelGroup *accel;
    char *titles[] = { "Filename", "Size", "Date" };
    char *wintitle;

    /* make the UI */
    ui = g_new0(Ui, 1);
    ui->cat = cat;

    accel = gtk_accel_group_new();

    ui->window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
    gtk_window_set_policy(GTK_WINDOW(ui->window), TRUE, TRUE, TRUE);
    wintitle = g_strconcat("Catalog - ", cat->catfile, NULL);
    gtk_window_set_title(GTK_WINDOW(ui->window), wintitle);
    g_free(wintitle);
    gtk_window_set_default_size(GTK_WINDOW(ui->window), 600, 350);
    gtk_signal_connect(GTK_OBJECT(ui->window), "destroy",
	    GTK_SIGNAL_FUNC(gtk_main_quit), NULL);

    /* main hbox */
    vbox = gtk_vbox_new(FALSE, UI_SPACING);
    gtk_container_add(GTK_CONTAINER(ui->window), vbox);

    /* menu */
    ui->menu = new_menu(accel, ui);
    gtk_box_pack_start(GTK_BOX(vbox), ui->menu, FALSE, FALSE, NO_SPACING);

    /* splitter */
    hpaned = gtk_hpaned_new();
    gtk_box_pack_start(GTK_BOX(vbox), hpaned, TRUE, TRUE, NO_SPACING);
    gtk_paned_set_handle_size(GTK_PANED(hpaned), 10);
    gtk_paned_set_gutter_size(GTK_PANED(hpaned), 10);

    /* left side ctree */
    scroll = gtk_scrolled_window_new(NULL, NULL);
    gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scroll),
	    GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
    gtk_paned_add1(GTK_PANED(hpaned), scroll);

    ui->dirtree = gtk_ctree_new(1, 0);
    gtk_widget_set_usize(GTK_WIDGET(ui->dirtree), 150, 450);
    gtk_clist_set_selection_mode(GTK_CLIST(ui->dirtree),
	    GTK_SELECTION_BROWSE);
    gtk_ctree_set_line_style(GTK_CTREE(ui->dirtree),
	    GTK_CTREE_LINES_DOTTED);
    gtk_clist_set_column_auto_resize(GTK_CLIST(ui->dirtree),
	    0, TRUE);
    gtk_signal_connect_after(GTK_OBJECT(ui->dirtree),
	    "tree_select_row",
	    GTK_SIGNAL_FUNC(cb_dirtree_select_row), ui);
    gtk_signal_connect_after(GTK_OBJECT(ui->dirtree),
	    "tree_expand",
	    GTK_SIGNAL_FUNC(cb_dirtree_expand_row), ui);
    gtk_signal_connect_after(GTK_OBJECT(ui->dirtree),
	    "tree_collapse",
	    GTK_SIGNAL_FUNC(cb_dirtree_collapse_row), ui);

    gtk_container_add(GTK_CONTAINER(scroll), ui->dirtree);

    /* right side clist */
    scroll = gtk_scrolled_window_new(NULL, NULL);
    gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scroll),
	    GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
    gtk_paned_add2(GTK_PANED(hpaned), scroll);

    ui->filelist = gtk_clist_new_with_titles(3, titles);
    gtk_clist_set_selection_mode(GTK_CLIST(ui->filelist),
	    GTK_SELECTION_BROWSE);
    gtk_clist_set_column_auto_resize(GTK_CLIST(ui->filelist), 0, TRUE);
    gtk_clist_set_column_auto_resize(GTK_CLIST(ui->filelist), 1, TRUE);
    gtk_clist_set_column_auto_resize(GTK_CLIST(ui->filelist), 2, TRUE);
    gtk_clist_set_sort_column(GTK_CLIST(ui->filelist), 0);
    gtk_clist_set_sort_type(GTK_CLIST(ui->filelist), GTK_SORT_ASCENDING);
    gtk_container_add(GTK_CONTAINER(scroll), ui->filelist);
    
    /* make various icons */
    new_icon(ui->dirtree, dir_xpm, &img_dir, &img_dir_mask);
    new_icon(ui->dirtree, disk_xpm, &img_disk, &img_disk_mask);
    new_icon(ui->dirtree, open_dir_xpm, &img_dir_open, &img_dir_open_mask);
    new_icon(ui->dirtree, open_disk_xpm, &img_disk_open, &img_disk_open_mask);
    new_icon(ui->dirtree, file_xpm, &img_file, &img_file_mask);

    ui->status = gtk_statusbar_new();
    gtk_box_pack_start(GTK_BOX(vbox), ui->status, FALSE, TRUE, NO_SPACING);
    
    gtk_widget_show_all(ui->window);
    gtk_window_add_accel_group(GTK_WINDOW(ui->window), accel);
    gtk_accel_group_lock(accel);

    update_statusbar(ui, SB_UPDATING);

    /* fill the left side tree */
    fill_disks(GTK_CTREE(ui->dirtree), cat);

    update_statusbar(ui, SB_READY);

    return ui;
}

void new_add_path(Ui *ui)
{
    GtkWidget *dialog;
    GtkWidget *vbox;
    GtkWidget *hbox;
    GtkWidget *label;
    GtkWidget *entry;
    GtkWidget *entry2;
    GtkWidget *button;
    GtkAccelGroup *accel;
    int key;

    dialog = gtk_dialog_new();
    accel = gtk_accel_group_new();
    gtk_window_set_title(GTK_WINDOW(dialog), "Add Path");
    gtk_window_set_position(GTK_WINDOW(dialog), GTK_WIN_POS_MOUSE);
    gtk_window_set_policy(GTK_WINDOW(dialog), FALSE, FALSE, TRUE);
    gtk_object_set_data(GTK_OBJECT(dialog), "ui", ui);

    /* main vbox */
    vbox = gtk_vbox_new(FALSE, UI_SPACING);
    gtk_container_set_border_width(GTK_CONTAINER(vbox), UI_SPACING);
    /* put it inside the dialog */
    gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), vbox, TRUE, TRUE, 0);

    /* Path: label */
    label = gtk_label_new("Path:");
    gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
    gtk_box_pack_start(GTK_BOX(vbox), label, TRUE, TRUE, 0);

    /* path entry */
    entry = gtk_entry_new();
    gtk_box_pack_start(GTK_BOX(vbox), entry, TRUE, TRUE, 0);
    gtk_object_set_data(GTK_OBJECT(dialog), "entry", entry);
    gtk_signal_connect(GTK_OBJECT(entry), "activate",
		       GTK_SIGNAL_FUNC(add_path_entry_cb), dialog);
    gtk_signal_connect(GTK_OBJECT(entry), "key_press_event",
		       GTK_SIGNAL_FUNC(dialog_keypress_cb), dialog);

    /* Title: label */
    label = gtk_label_new("Disk Label:");
    gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
    gtk_box_pack_start(GTK_BOX(vbox), label, TRUE, TRUE, 0);

    /* disk label */
    entry2 = gtk_entry_new();
    gtk_box_pack_start(GTK_BOX(vbox), entry2, TRUE, TRUE, 0);
    gtk_object_set_data(GTK_OBJECT(dialog), "label", entry2);
    gtk_signal_connect(GTK_OBJECT(entry2), "activate",
		       GTK_SIGNAL_FUNC(add_path_entry_cb), dialog);
    gtk_signal_connect(GTK_OBJECT(entry2), "key_press_event",
		       GTK_SIGNAL_FUNC(dialog_keypress_cb), dialog);

    /* bottom part */
    gtk_container_set_border_width(GTK_CONTAINER
				   (GTK_DIALOG(dialog)->action_area),
				   UI_SPACING);
    hbox = gtk_hbox_new(TRUE, UI_SPACING);
    gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->action_area), hbox,
		       TRUE, TRUE, 0);

    button = gtk_button_new_with_label("");
    key = gtk_label_parse_uline(GTK_LABEL(GTK_BIN(button)->child),
	    "_OK");
    gtk_box_pack_start(GTK_BOX(hbox), button, TRUE, TRUE, 0);
    gtk_signal_connect(GTK_OBJECT(button), "clicked",
		       GTK_SIGNAL_FUNC(add_path_entry_cb), dialog);
    gtk_widget_add_accelerator(button, "clicked", accel,
                              key, GDK_MOD1_MASK, 0);

    button = gtk_button_new_with_label("");
    key = gtk_label_parse_uline(GTK_LABEL(GTK_BIN(button)->child),
	    "_Cancel");
    gtk_box_pack_start(GTK_BOX(hbox), button, TRUE, TRUE, 0);
    gtk_signal_connect_object(GTK_OBJECT(button), "clicked",
		       gtk_widget_destroy, (gpointer)dialog);
    gtk_widget_add_accelerator(button, "clicked", accel,
                              key, GDK_MOD1_MASK, 0);

    gtk_signal_connect(GTK_OBJECT(dialog), "key_press_event",
		       GTK_SIGNAL_FUNC(dialog_keypress_cb), dialog);

    gtk_widget_show_all(dialog);
    gtk_widget_grab_focus(entry);
    gtk_window_add_accel_group(GTK_WINDOW(dialog), accel);
}

void new_add_cd(Ui *ui)
{
    GtkWidget *dialog;
    GtkWidget *vbox;
    GtkWidget *hbox;
    GtkWidget *label;
    GtkWidget *entry;
    GtkWidget *button;
    GtkAccelGroup *accel;
    int key;

    dialog = gtk_dialog_new();
    accel = gtk_accel_group_new();
    gtk_window_set_title(GTK_WINDOW(dialog), "Add CDROM");
    gtk_window_set_position(GTK_WINDOW(dialog), GTK_WIN_POS_MOUSE);
    gtk_window_set_policy(GTK_WINDOW(dialog), FALSE, FALSE, TRUE);
    gtk_object_set_data(GTK_OBJECT(dialog), "ui", ui);

    /* main vbox */
    vbox = gtk_vbox_new(FALSE, UI_SPACING);
    gtk_container_set_border_width(GTK_CONTAINER(vbox), UI_SPACING);
    /* put it inside the dialog */
    gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), vbox, TRUE, TRUE, 0);

    /* Disk label */
    label = gtk_label_new("Disk Label:");
    gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
    gtk_box_pack_start(GTK_BOX(vbox), label, TRUE, TRUE, 0);

    /* label entry */
    entry = gtk_entry_new();
    gtk_box_pack_start(GTK_BOX(vbox), entry, TRUE, TRUE, 0);
    gtk_object_set_data(GTK_OBJECT(dialog), "label", entry);
    gtk_signal_connect(GTK_OBJECT(entry), "activate",
		       GTK_SIGNAL_FUNC(add_cd_entry_cb), dialog);
    gtk_signal_connect(GTK_OBJECT(entry), "key_press_event",
		       GTK_SIGNAL_FUNC(dialog_keypress_cb), dialog);

    /* bottom part */
    gtk_container_set_border_width(GTK_CONTAINER
				   (GTK_DIALOG(dialog)->action_area),
				   UI_SPACING);
    hbox = gtk_hbox_new(TRUE, UI_SPACING);
    gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->action_area), hbox,
		       TRUE, TRUE, 0);

    button = gtk_button_new_with_label("");
    key = gtk_label_parse_uline(GTK_LABEL(GTK_BIN(button)->child),
	    "_OK");
    gtk_box_pack_start(GTK_BOX(hbox), button, TRUE, TRUE, 0);
    gtk_signal_connect(GTK_OBJECT(button), "clicked",
		       GTK_SIGNAL_FUNC(add_cd_entry_cb), dialog);
    gtk_widget_add_accelerator(button, "clicked", accel,
                              key, GDK_MOD1_MASK, 0);

    button = gtk_button_new_with_label("");
    key = gtk_label_parse_uline(GTK_LABEL(GTK_BIN(button)->child),
	    "_Cancel");
    gtk_box_pack_start(GTK_BOX(hbox), button, TRUE, TRUE, 0);
    gtk_signal_connect_object(GTK_OBJECT(button), "clicked",
		       gtk_widget_destroy, (gpointer)dialog);
    gtk_widget_add_accelerator(button, "clicked", accel,
                              key, GDK_MOD1_MASK, 0);

    gtk_signal_connect(GTK_OBJECT(dialog), "key_press_event",
		       GTK_SIGNAL_FUNC(dialog_keypress_cb), dialog);

    gtk_widget_show_all(dialog);
    gtk_widget_grab_focus(entry);
    gtk_window_add_accel_group(GTK_WINDOW(dialog), accel);
}

void new_find(Ui *ui)
{
    GtkWidget *dialog;
    GtkWidget *vbox;
    GtkWidget *hbox;
    GtkWidget *label;
    GtkWidget *entry;
    GtkWidget *sens;
    GtkWidget *button;

    GtkWidget *option;
    GtkWidget *menu;
    GtkWidget *item;

    GtkWidget *scroll;

    GtkAccelGroup *accel;
    int key;
    char *titles[] = { "File Name", "Disk", "Path", "Size", "Date" };

    dialog = gtk_dialog_new();
    accel = gtk_accel_group_new();
    gtk_window_set_title(GTK_WINDOW(dialog), "Find File");
    gtk_window_set_position(GTK_WINDOW(dialog), GTK_WIN_POS_MOUSE);
    gtk_window_set_policy(GTK_WINDOW(dialog), FALSE, TRUE, TRUE);
    gtk_object_set_data(GTK_OBJECT(dialog), "ui", ui);

    /* main vbox */
    vbox = gtk_vbox_new(FALSE, UI_SPACING);
    gtk_container_set_border_width(GTK_CONTAINER(vbox), UI_SPACING);
    /* put it inside the dialog */
    gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), vbox, TRUE, TRUE, 0);

    /* Find: label */
    label = gtk_label_new("");
    key = gtk_label_parse_uline(GTK_LABEL(label),
	    "_Filename Pattern:");
    gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
    gtk_box_pack_start(GTK_BOX(vbox), label, FALSE, TRUE, 0);

    /* find entry */
    entry = gtk_entry_new();
    gtk_box_pack_start(GTK_BOX(vbox), entry, FALSE, TRUE, 0);
    gtk_object_set_data(GTK_OBJECT(dialog), "entry", entry);
    gtk_signal_connect(GTK_OBJECT(entry), "activate",
		       GTK_SIGNAL_FUNC(find_entry_cb), dialog);
    gtk_signal_connect(GTK_OBJECT(entry), "key_press_event",
		       GTK_SIGNAL_FUNC(dialog_keypress_cb), dialog);
    gtk_widget_add_accelerator(entry, "grab_focus", accel, key, GDK_MOD1_MASK, 0);

    /* modifiers */
    hbox = gtk_hbox_new(FALSE, UI_SPACING);
    gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, TRUE, NO_SPACING);

    /* sensitive */
    sens = gtk_check_button_new_with_label("");
    key = gtk_label_parse_uline(GTK_LABEL(GTK_BIN(sens)->child),
	    "Case _Sensitive");
    gtk_object_set_data(GTK_OBJECT(dialog), "toggle", sens);
    gtk_box_pack_start(GTK_BOX(hbox), sens, TRUE, TRUE, NO_SPACING);
    gtk_signal_connect_object(GTK_OBJECT(sens), "clicked",
			      GTK_SIGNAL_FUNC(gtk_widget_grab_focus),
			      (gpointer) entry);
    gtk_widget_add_accelerator(sens, "clicked", accel,
                              key, GDK_MOD1_MASK, 0);

    option = gtk_option_menu_new();
    gtk_object_set_data(GTK_OBJECT(dialog), "option", option);
    gtk_box_pack_start(GTK_BOX(hbox), option, TRUE, TRUE, NO_SPACING);

    menu = gtk_menu_new();
    item = gtk_menu_item_new_with_label("Shell Glob Pattern");
    gtk_menu_append(GTK_MENU(menu), item);
    item = gtk_menu_item_new_with_label("Regex Pattern");
    gtk_menu_append(GTK_MENU(menu), item);
    item = gtk_menu_item_new_with_label("Exact Match");
    gtk_menu_append(GTK_MENU(menu), item);
    gtk_option_menu_set_menu(GTK_OPTION_MENU(option), menu);

    /* find results list */
    scroll = gtk_scrolled_window_new(NULL, NULL);
    gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scroll),
	    GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
    gtk_box_pack_start(GTK_BOX(vbox), scroll, TRUE, TRUE, NO_SPACING);

    ui->findlist = gtk_clist_new_with_titles(5, titles);
    gtk_widget_set_usize(GTK_WIDGET(ui->findlist), 580, 200);
    gtk_clist_set_column_auto_resize(GTK_CLIST(ui->findlist), 0, TRUE);
    gtk_clist_set_column_auto_resize(GTK_CLIST(ui->findlist), 1, TRUE);
    gtk_clist_set_column_auto_resize(GTK_CLIST(ui->findlist), 2, TRUE);
    gtk_clist_set_column_auto_resize(GTK_CLIST(ui->findlist), 3, TRUE);
    gtk_clist_set_column_auto_resize(GTK_CLIST(ui->findlist), 4, TRUE);
    gtk_clist_set_selection_mode(GTK_CLIST(ui->findlist),
	    GTK_SELECTION_BROWSE);
    gtk_clist_set_auto_sort(GTK_CLIST(ui->findlist), TRUE);
    gtk_clist_set_sort_column(GTK_CLIST(ui->findlist), 0);
    gtk_signal_connect(GTK_OBJECT(ui->findlist), "key_press_event",
		       GTK_SIGNAL_FUNC(dialog_keypress_cb), dialog);
    gtk_container_add(GTK_CONTAINER(scroll), ui->findlist);

    /* bottom part */
    gtk_container_set_border_width(GTK_CONTAINER
				   (GTK_DIALOG(dialog)->action_area),
				   UI_SPACING);
    hbox = gtk_hbox_new(TRUE, UI_SPACING);
    gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->action_area), hbox,
		       TRUE, TRUE, 0);

    button = gtk_button_new_with_label("Find");
    gtk_box_pack_start(GTK_BOX(hbox), button, TRUE, TRUE, 0);
    gtk_signal_connect(GTK_OBJECT(button), "clicked",
		       GTK_SIGNAL_FUNC(find_entry_cb), dialog);

    button = gtk_button_new_with_label("");
    key = gtk_label_parse_uline(GTK_LABEL(GTK_BIN(button)->child),
	    "_Close");
    gtk_box_pack_start(GTK_BOX(hbox), button, TRUE, TRUE, NO_SPACING);
    gtk_signal_connect_object(GTK_OBJECT(button), "clicked",
		       gtk_widget_destroy, (gpointer)dialog);
    gtk_widget_add_accelerator(button, "clicked", accel,
                              key, GDK_MOD1_MASK, 0);

    gtk_widget_show_all(dialog);
    gtk_widget_grab_focus(entry);
    gtk_window_add_accel_group(GTK_WINDOW(dialog), accel);
}

static void new_icon(GtkWidget *widget, char **data, GdkPixmap **pixmap,
	GdkBitmap **mask)
{
    GdkColormap *colormap;

    colormap = gtk_widget_get_colormap(widget);
    *pixmap = gdk_pixmap_colormap_create_from_xpm_d(NULL, colormap,
	    mask, NULL, data);
}

static GtkItemFactoryEntry menu[] = {
    { "/_File", NULL, NULL, 0, "<Branch>" },
    { "/File/_Save", "<alt>S", menu_cb, 11, NULL },
    { "/File/E_xit", "<alt>X", menu_cb, 19, NULL },

    { "/_Catalog", NULL, NULL, 0, "<Branch>" },
    { "/Catalog/_Add CD", "<alt>A", menu_cb, 21, NULL },
    { "/Catalog/Add _Path", "<alt>P", menu_cb, 22, NULL },
    { "/Catalog/_Search Disks", "F7", menu_cb, 23, NULL },
    { "/Catalog/sep1", NULL, NULL, 0, "<Separator>", },
    { "/Catalog/_Delete Disk", "<alt>D", menu_cb, 31, NULL },

    { "/_Help", NULL, NULL, 0, "<LastBranch>" },
    { "/Help/_About", NULL, NULL, 0, NULL }
};

static GtkWidget *new_menu(GtkAccelGroup *accel, Ui *ui)
{
    GtkItemFactory *fact;
    int count = sizeof(menu) / sizeof(menu[0]);

    fact = gtk_item_factory_new(GTK_TYPE_MENU_BAR, "<main>", accel);
    gtk_item_factory_create_items(fact, count, menu, ui);
    ui->fact = fact;
    return gtk_item_factory_get_widget(fact, "<main>");
}
